<?php
/**
 * Plugin Name: DMG Query Loop Block Enhancements
 * Plugin URI:  https://github.com/ANMedia/dmg-query-loop-block-enhancements-wp-plugin
 * Description: Enhancements for the core Query Loop block
 * Version:     1.0.0
 * Author:      Metro.co.uk
 * Author URI:  https://github.com/ANMedia/dmg-query-loop-block-enhancements-wp-plugin/graphs/contributors
 */

namespace DMG;

/**
 * Class Query_Loop_Block_Enhancements
 *
 * Register all scripts necessary for query loop customizations and add filters
 *
 * @subpackage DMG
 */
class Query_Loop_Block_Enhancements {

	/**
	 * Init
	 */
	public static function init() {
		add_action( 'enqueue_block_editor_assets', array( __CLASS__, 'enqueue_assets' ) );
		add_filter( 'query_loop_block_query_vars', array( __CLASS__, 'filter_block_query' ), 10, 2 );
		add_action( 'pre_get_posts', array( __CLASS__, 'filter_multi_post_types_query_var' ) );
	}

	/**
	 * Enqueue block editor assets
	 */
	public static function enqueue_assets() {
		$handle = 'dmg-query-loop-block-enhancements';

		// Include asset file for dependencies
		$asset_file = include( plugin_dir_path( __FILE__ ) . 'build/index.asset.php');

		wp_enqueue_script(
			$handle,
			plugins_url( 'build/index.js', __FILE__ ),
			$asset_file['dependencies'],
			$asset_file['version']
		);

		wp_enqueue_style(
			$handle,
			plugins_url( 'build/index.css', __FILE__ ),
			array(),
			$asset_file['version']
		);
	}

	/**
	 * Check for curated posts and configure the query accordingly
	 *
	 * @param array    $query Query arguments
	 * @param WP_Block $block The block instance
	 * @return array The Updated query arguments
	 */
	public static function filter_block_query( $query, $block ) {

		// Check the include property
		$curated_posts = ( ! empty( $block->context['query']['include'] ) ) ? $block->context['query']['include'] : '';

		// Separate into post IDs
		$post_ids = explode( ',', $curated_posts );

		if ( empty( $post_ids ) ) {
			return $query;
		}

		// Set proper query vars
		$query['orderby']        = 'post__in';
		$query['post__in']       = $post_ids;
		$query['posts_per_page'] = count( $post_ids );
		$query['post_type']      = 'any';

		return $query;
	}

	/**
	 * Handling for query vars indicating selection of all post types
	 * 
	 * @param WP_Query $query The query object
	 */
	public static function filter_multi_post_types_query_var( $query ) {

		// Check for query var and set post type to any
		if ( ! empty( $_GET['dmg_query_loop_block_enhancements_post_curator'] ) ) {
			$query->set( 'post_type', 'any' );
			$query->set( 'post_status', array( 'publish', 'future' ) );
		}

		return $query;
	}
}

Query_Loop_Block_Enhancements::init();
