/**
 * WordPress Dependencies
 */
import { useState } from '@wordpress/element';

/**
 * The `SortableList` component
 * 
 * A component for allowing drag-and-drop sorting of arbitrary items
 */
export const SortableList = (({ items, onChange }) => {

	// Set the states
	const [ itemBeingDragged, setItemBeingDragged ] = useState();

	// Callback for when dragging starts
	const onDragStart = (e, item) => {
		e.target.classList.add('dragging');
		setItemBeingDragged(item);
	}

	// Callback for when dragging starts
	const onDragEnd = (e, item) => {
		e.target.classList.remove('dragging');
		setItemBeingDragged();
	}

	// Callback for when item dropped
	const onDrop = (e, itemDropped) => {

		// Remove the dragging class
		e.target.classList.remove('dragging');

		// Remove the over class form the item being dropped
		e.target.classList.remove('over');

		// Deep clone items after filtering out the item being dragged
		let clonedItems = JSON.parse(JSON.stringify(items.filter((item) => item.data !== itemBeingDragged.data)));

		// Determine the index of the drop element
		const index = Array.prototype.indexOf.call(e.target.parentElement.childNodes, e.target);

		// Splice in the dragged element at the new index
		clonedItems.splice(index, 0, itemBeingDragged);

		// Fire onChange event
		onChange(clonedItems);
	}

	// Remove an item
	const removeItem = (itemToRemove) => {
		onChange(items.filter((item) => item.data !== itemToRemove.data));
	}

	return (
		<ul className="dmg-query-loop-block-enhancements-sortable-list">
			{items.map((item) => {
				return (
					<li
						className="dmg-query-loop-block-enhancements-sortable-list__item"
						draggable="true"
						onDragStart={ (e) => onDragStart(e, item) }
						onDragEnd={ (e) => onDragEnd(e, item) }
						onDragEnter={(e) => e.target.classList.add('over')}
						onDragLeave={(e) => e.target.classList.remove('over')}
						onDrop={ (e) => onDrop(e, item) }
					>
						{typeof(item.content) === 'string' &&
							item.content
						}
						{typeof(item.content) === 'function' &&
							item.content()
						}
						<span
							className="dashicons dashicons-remove"
							onClick={ () => removeItem(item) }
						>
						</span>
					</li>
				);
			})}
		</ul>
	);
});